@@ -41,7 +41,7 @@ from typing import Optional, TYPE_CHECKING, Dict, List
 
 from .import util, ecc
 from .util import (bfh, bh2u, format_satoshis, json_decode, json_normalize,
-                   is_hash256_str, is_hex_str, to_bytes)
+                   is_hash256_str, is_hex_str, to_bytes, parse_max_spend)
 from . import bitcoin
 from .bitcoin import is_address,  hash_160, COIN
 from .bip32 import BIP32Node
@@ -50,7 +50,7 @@ from .transaction import (Transaction, multisig_script, TxOutput, PartialTransac
                           tx_from_any, PartialTxInput, TxOutpoint)
 from .invoices import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
 from .synchronizer import Notifier
-from .wallet import Abstract_Wallet, create_new_wallet, restore_wallet_from_text, Deterministic_Wallet
+from .wallet import Abstract_Wallet, create_new_wallet, restore_wallet_from_text, Deterministic_Wallet, BumpFeeStrategy
 from .address_synchronizer import TX_HEIGHT_LOCAL
 from .mnemonic import Mnemonic
 from .lnutil import SENT, RECEIVED
@@ -77,7 +77,7 @@ class NotSynchronizedException(Exception):
 
 
 def satoshis_or_max(amount):
-    return satoshis(amount) if amount != '!' else '!'
+    return satoshis(amount) if not parse_max_spend(amount) else amount
 
 def satoshis(amount):
     # satoshi conversion must not be performed by the parser
@@ -93,6 +93,7 @@ class Command:
         self.requires_network = 'n' in s
         self.requires_wallet = 'w' in s
         self.requires_password = 'p' in s
+        self.requires_lightning = 'l' in s
         self.description = func.__doc__
         self.help = self.description.split('.')[0] if self.description else None
         varnames = func.__code__.co_varnames[1:func.__code__.co_argcount]
@@ -145,6 +146,8 @@ def command(s):
                 raise Exception('wallet not loaded')
             if cmd.requires_password and password is None and wallet.has_password():
                 raise Exception('Password required')
+            if cmd.requires_lightning and (not wallet or not wallet.has_lightning()):
+                raise Exception('Lightning not enabled in this wallet')
             return await func(*args, **kwargs)
         return func_wrapper
     return decorator
@@ -214,7 +217,6 @@ class Commands:
     @command('n')
     async def stop(self):
         """Stop daemon"""
-        # TODO it would be nice if this could stop the GUI too
         await self.daemon.stop()
         return "Daemon stopped"
 
@@ -236,7 +238,7 @@ class Commands:
     @command('n')
     async def close_wallet(self, wallet_path=None):
         """Close wallet"""
-        return self.daemon.stop_wallet(wallet_path)
+        return await self.daemon._stop_wallet(wallet_path)
 
     @command('')
     async def create(self, passphrase=None, password=None, encrypt_file=True, seed_type=None, wallet_path=None):
@@ -707,7 +709,38 @@ class Commands:
 
         return json_normalize(wallet.get_detailed_history(**kwargs))
 
-    @command('w')
+    @command('wp')
+    async def bumpfee(self, tx, new_fee_rate, from_coins=None, strategies=None, password=None, unsigned=False, wallet: Abstract_Wallet = None):
+        """ Bump the Fee for an unconfirmed Transaction """
+        tx = Transaction(tx)
+        domain_coins = from_coins.split(',') if from_coins else None
+        coins = wallet.get_spendable_coins(None)
+        if domain_coins is not None:
+            coins = [coin for coin in coins if (coin.prevout.to_str() in domain_coins)]
+        strategies = strategies.split(',') if strategies else None
+        bumpfee_strategies = None
+        if strategies is not None:
+            bumpfee_strategies = []
+            for strategy in strategies:
+                if strategy == 'CoinChooser':
+                    bumpfee_strategies.append(BumpFeeStrategy.COINCHOOSER)
+                elif strategy == 'DecreaseChange':
+                    bumpfee_strategies.append(BumpFeeStrategy.DECREASE_CHANGE)
+                elif strategy == 'DecreasePayment':
+                    bumpfee_strategies.append(BumpFeeStrategy.DECREASE_PAYMENT)
+                else:
+                    raise Exception("Invalid Choice of Strategies")
+        new_tx = wallet.bump_fee(
+            tx=tx,
+            txid=tx.txid(),
+            coins=coins,
+            strategies=bumpfee_strategies,
+            new_fee_rate=new_fee_rate)
+        if not unsigned:
+            wallet.sign_transaction(new_tx, password)
+        return new_tx.serialize()
+
+    @command('wl')
     async def lightning_history(self, show_fiat=False, wallet: Abstract_Wallet = None):
         """ lightning history """
         lightning_history = wallet.lnworker.get_history() if wallet.lnworker else []
@@ -882,14 +915,12 @@ class Commands:
         expiration = int(expiration) if expiration else None
         req = wallet.make_payment_request(addr, amount, memo, expiration)
         wallet.add_payment_request(req)
-        wallet.save_db()
         return wallet.export_request(req)
 
-    @command('wn')
+    @command('wnl')
     async def add_lightning_request(self, amount, memo='', expiration=3600, wallet: Abstract_Wallet = None):
         amount_sat = int(satoshis(amount))
         key = await wallet.lnworker._add_request_coro(amount_sat, memo, expiration)
-        wallet.save_db()
         return wallet.get_formatted_request(key)
 
     @command('w')
@@ -913,9 +944,7 @@ class Commands:
     @command('w')
     async def rmrequest(self, address, wallet: Abstract_Wallet = None):
         """Remove a payment request"""
-        result = wallet.remove_payment_request(address)
-        wallet.save_db()
-        return result
+        return wallet.remove_payment_request(address)
 
     @command('w')
     async def clear_requests(self, wallet: Abstract_Wallet = None):
@@ -999,13 +1028,13 @@ class Commands:
         return sorted(known_commands.keys())
 
     # lightning network commands
-    @command('wn')
+    @command('wnl')
     async def add_peer(self, connection_string, timeout=20, gossip=False, wallet: Abstract_Wallet = None):
         lnworker = self.network.lngossip if gossip else wallet.lnworker
         await lnworker.add_peer(connection_string)
         return True
 
-    @command('wn')
+    @command('wnl')
     async def list_peers(self, gossip=False, wallet: Abstract_Wallet = None):
         lnworker = self.network.lngossip if gossip else wallet.lnworker
         return [{
@@ -1016,7 +1045,7 @@ class Commands:
             'channels': [c.funding_outpoint.to_str() for c in p.channels.values()],
         } for p in lnworker.peers.values()]
 
-    @command('wpn')
+    @command('wpnl')
     async def open_channel(self, connection_string, amount, push_amount=0, password=None, wallet: Abstract_Wallet = None):
         funding_sat = satoshis(amount)
         push_sat = satoshis(push_amount)
@@ -1040,7 +1069,7 @@ class Commands:
         invoice = LNInvoice.from_bech32(invoice)
         return invoice.to_debug_json()
 
-    @command('wn')
+    @command('wnl')
     async def lnpay(self, invoice, attempts=1, timeout=30, wallet: Abstract_Wallet = None):
         lnworker = wallet.lnworker
         lnaddr = lnworker._check_invoice(invoice)
@@ -1054,12 +1083,12 @@ class Commands:
             'log': [x.formatted_tuple() for x in log]
         }
 
-    @command('w')
+    @command('wl')
     async def nodeid(self, wallet: Abstract_Wallet = None):
         listen_addr = self.config.get('lightning_listen')
         return bh2u(wallet.lnworker.node_keypair.pubkey) + (('@' + listen_addr) if listen_addr else '')
 
-    @command('w')
+    @command('wl')
     async def list_channels(self, wallet: Abstract_Wallet = None):
         # FIXME: we need to be online to display capacity of backups
         from .lnutil import LOCAL, REMOTE, format_short_channel_id
@@ -1091,7 +1120,7 @@ class Commands:
             } for channel_id, chan in backups
         ]
 
-    @command('wn')
+    @command('wnl')
     async def dumpgraph(self, wallet: Abstract_Wallet = None):
         return wallet.lnworker.channel_db.to_dict()
 
@@ -1101,7 +1130,7 @@ class Commands:
         fee_est = ast.literal_eval(fees)
         self.network.update_fee_estimates(fee_est=fee_est)
 
-    @command('wn')
+    @command('wnl')
     async def enable_htlc_settle(self, b: bool, wallet: Abstract_Wallet = None):
         wallet.lnworker.enable_htlc_settle = b
 
@@ -1120,14 +1149,14 @@ class Commands:
         l = wallet.get_invoices()
         return [wallet.export_invoice(x) for x in l]
 
-    @command('wn')
+    @command('wnl')
     async def close_channel(self, channel_point, force=False, wallet: Abstract_Wallet = None):
         txid, index = channel_point.split(':')
         chan_id, _ = channel_id_from_funding_tx(txid, int(index))
         coro = wallet.lnworker.force_close_channel(chan_id) if force else wallet.lnworker.close_channel(chan_id)
         return await coro
 
-    @command('wn')
+    @command('wnl')
     async def request_force_close(self, channel_point, connection_string=None, wallet: Abstract_Wallet = None):
         """
         Requests the remote to force close a channel.
@@ -1138,17 +1167,17 @@ class Commands:
         chan_id, _ = channel_id_from_funding_tx(txid, int(index))
         await wallet.lnworker.request_force_close(chan_id, connect_str=connection_string)
 
-    @command('w')
+    @command('wl')
     async def export_channel_backup(self, channel_point, wallet: Abstract_Wallet = None):
         txid, index = channel_point.split(':')
         chan_id, _ = channel_id_from_funding_tx(txid, int(index))
         return wallet.lnworker.export_channel_backup(chan_id)
 
-    @command('w')
+    @command('wl')
     async def import_channel_backup(self, encrypted, wallet: Abstract_Wallet = None):
         return wallet.lnworker.import_channel_backup(encrypted)
 
-    @command('wn')
+    @command('wnl')
     async def get_channel_ctx(self, channel_point, iknowwhatimdoing=False, wallet: Abstract_Wallet = None):
         """ return the current commitment transaction of a channel """
         if not iknowwhatimdoing:
@@ -1160,12 +1189,12 @@ class Commands:
         tx = chan.force_close_tx()
         return tx.serialize()
 
-    @command('wn')
+    @command('wnl')
     async def get_watchtower_ctn(self, channel_point, wallet: Abstract_Wallet = None):
         """ return the local watchtower's ctn of channel. used in regtests """
         return await self.network.local_watchtower.sweepstore.get_ctn(channel_point, None)
 
-    @command('wnp')
+    @command('wnpl')
     async def normal_swap(self, onchain_amount, lightning_amount, password=None, wallet: Abstract_Wallet = None):
         """
         Normal submarine swap: send on-chain BTC, receive on Lightning
@@ -1196,7 +1225,7 @@ class Commands:
             'onchain_amount': format_satoshis(onchain_amount_sat),
         }
 
-    @command('wn')
+    @command('wnl')
     async def reverse_swap(self, lightning_amount, onchain_amount, wallet: Abstract_Wallet = None):
         """Reverse submarine swap: send on Lightning, receive on-chain
         """
@@ -1303,6 +1332,8 @@ command_options = {
     'iknowwhatimdoing': (None, "Acknowledge that I understand the full implications of what I am about to do"),
     'gossip':      (None, "Apply command to gossip node instead of wallet"),
     'connection_string':      (None, "Lightning network node ID or network address"),
+    'new_fee_rate': (None, "The Updated/Increased Transaction fee rate (in sat/byte)"),
+    'strategies': (None, "Select RBF any one or multiple RBF strategies in any order, separated by ','; Options : 'CoinChooser','DecreaseChange','DecreasePayment' "),
 }
 
 
@@ -1322,7 +1353,7 @@ arg_types = {
     'inputs': json_loads,
     'outputs': json_loads,
     'fee': lambda x: str(Decimal(x)) if x is not None else None,
-    'amount': lambda x: str(Decimal(x)) if x != '!' else '!',
+    'amount': lambda x: str(Decimal(x)) if not parse_max_spend(x) else x,
     'locktime': int,
     'addtransaction': eval_bool,
     'fee_method': str,
@@ -1349,7 +1380,7 @@ def set_default_subparser(self, name, args=None):
     """see http://stackoverflow.com/questions/5176691/argparse-how-to-specify-a-default-subcommand"""
     subparser_found = False
     for arg in sys.argv[1:]:
-        if arg in ['-h', '--help']:  # global help if no subparser
+        if arg in ['-h', '--help', '--version']:  # global help/version if no subparser
             break
     else:
         for x in self._subparsers._actions:
@@ -1415,6 +1446,7 @@ def add_global_options(parser):
     group.add_argument("--testnet", action="store_true", dest="testnet", default=False, help="Use Testnet")
     group.add_argument("--regtest", action="store_true", dest="regtest", default=False, help="Use Regtest")
     group.add_argument("--simnet", action="store_true", dest="simnet", default=False, help="Use Simnet")
+    group.add_argument("--signet", action="store_true", dest="signet", default=False, help="Use Signet")
     group.add_argument("-o", "--offline", action="store_true", dest="offline", default=False, help="Run offline")
 
 def add_wallet_option(parser):
@@ -1425,6 +1457,7 @@ def get_parser():
     # create main parser
     parser = argparse.ArgumentParser(
         epilog="Run 'electrum help <command>' to see the help for a command")
+    parser.add_argument("--version", dest="cmd", action='store_const', const='version', help="Return the version of Electrum.")
     add_global_options(parser)
     add_wallet_option(parser)
     subparsers = parser.add_subparsers(dest='cmd', metavar='<command>')
@@ -1442,6 +1475,8 @@ def get_parser():
     # daemon
     parser_daemon = subparsers.add_parser('daemon', help="Run Daemon")
     parser_daemon.add_argument("-d", "--detached", action="store_true", dest="detach", default=False, help="run daemon in detached mode")
+    parser_daemon.add_argument("--rpcsock", dest="rpcsock", default=None, help="what socket type to which to bind RPC daemon", choices=['unix', 'tcp', 'auto'])
+    parser_daemon.add_argument("--rpcsockpath", dest="rpcsockpath", help="where to place RPC file socket")
     add_network_options(parser_daemon)
     add_global_options(parser_daemon)
     # commands
