@@ -127,7 +127,14 @@ class KeyStore(Logger, ABC):
         pass
 
     @abstractmethod
-    def sign_message(self, sequence: 'AddressIndexGeneric', message, password) -> bytes:
+    def sign_message(
+            self,
+            sequence: 'AddressIndexGeneric',
+            message: str,
+            password,
+            *,
+            script_type: Optional[str] = None,
+    ) -> bytes:
         pass
 
     @abstractmethod
@@ -175,7 +182,7 @@ class Software_KeyStore(KeyStore):
     def may_have_password(self):
         return not self.is_watching_only()
 
-    def sign_message(self, sequence, message, password) -> bytes:
+    def sign_message(self, sequence, message, password, *, script_type=None) -> bytes:
         privkey, compressed = self.get_private_key(sequence, password)
         key = ecc.ECPrivkey(privkey)
         return key.sign_message(message, compressed)
@@ -911,12 +918,11 @@ def bip39_is_checksum_valid(
     return checksum == calculated_checksum, True
 
 
-def from_bip39_seed(seed, passphrase, derivation, xtype=None):
+def from_bip43_rootseed(root_seed, derivation, xtype=None):
     k = BIP32_KeyStore({})
-    bip32_seed = bip39_to_seed(seed, passphrase)
     if xtype is None:
         xtype = xtype_from_derivation(derivation)
-    k.add_xprv_from_seed(bip32_seed, xtype, derivation)
+    k.add_xprv_from_seed(root_seed, xtype, derivation)
     return k
 
 
