@@ -40,12 +40,12 @@ import copy
 import functools
 
 import aiorpcx
-from aiorpcx import TaskGroup, ignore_after
+from aiorpcx import ignore_after
 from aiohttp import ClientResponse
 
 from . import util
-from .util import (log_exceptions, ignore_exceptions,
-                   bfh, SilentTaskGroup, make_aiohttp_session, send_exception_to_crash_reporter,
+from .util import (log_exceptions, ignore_exceptions, OldTaskGroup,
+                   bfh, make_aiohttp_session, send_exception_to_crash_reporter,
                    is_hash256_str, is_non_negative_integer, MyEncoder, NetworkRetryManager,
                    nullcontext)
 from .bitcoin import COIN
@@ -246,7 +246,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
     LOGGING_SHORTCUT = 'n'
 
-    taskgroup: Optional[TaskGroup]
+    taskgroup: Optional[OldTaskGroup]
     interface: Optional[Interface]
     interfaces: Dict[ServerAddr, Interface]
     _connecting_ifaces: Set[ServerAddr]
@@ -437,7 +437,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
     def is_connected(self):
         interface = self.interface
-        return interface is not None and interface.ready.done()
+        return interface is not None and interface.is_connected_and_ready()
 
     def is_connecting(self):
         return self.connection_status == 'connecting'
@@ -462,7 +462,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         async def get_relay_fee():
             self.relay_fee = await interface.get_relay_fee()
 
-        async with TaskGroup() as group:
+        async with OldTaskGroup() as group:
             await group.spawn(get_banner)
             await group.spawn(get_donation_address)
             await group.spawn(get_server_peers)
@@ -707,11 +707,19 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
         i = self.interfaces[server]
         if old_interface != i:
+            if not i.is_connected_and_ready():
+                return
             self.logger.info(f"switching to {server}")
-            assert i.ready.done(), "interface we are switching to is not ready yet"
             blockchain_updated = i.blockchain != self.blockchain()
             self.interface = i
-            await i.taskgroup.spawn(self._request_server_info(i))
+            try:
+                await i.taskgroup.spawn(self._request_server_info(i))
+            except RuntimeError as e:  # see #7677
+                if len(e.args) >= 1 and e.args[0] == 'task group terminated':
+                    self.logger.warning(f"tried to switch to {server} but interface.taskgroup is already dead.")
+                    self.interface = None
+                    return
+                raise
             util.trigger_callback('default_server_changed')
             self.default_server_changed_event.set()
             self.default_server_changed_event.clear()
@@ -758,6 +766,8 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         util.trigger_callback('network_updated')
 
     def get_network_timeout_seconds(self, request_type=NetworkTimeout.Generic) -> int:
+        if self.config.get('network_timeout', None):
+            return int(self.config.get('network_timeout'))
         if self.oneserver and not self.auto_connect:
             return request_type.MOST_RELAXED
         if self.proxy:
@@ -839,7 +849,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
                 assert iface.ready.done(), "interface not ready yet"
                 # try actual request
                 try:
-                    async with TaskGroup(wait=any) as group:
+                    async with OldTaskGroup(wait=any) as group:
                         task = await group.spawn(func(self, *args, **kwargs))
                         await group.spawn(iface.got_disconnected.wait())
                 except RequestTimedOut:
@@ -907,27 +917,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         # server_msg is untrusted input so it should not be shown to the user. see #4968
         server_msg = str(server_msg)
         server_msg = server_msg.replace("\n", r"\n")
-        # https://github.com/bitcoin/bitcoin/blob/5bb64acd9d3ced6e6f95df282a1a0f8b98522cb0/src/policy/policy.cpp
-        # grep "reason ="
-        policy_error_messages = {
-            r"version": _("Transaction uses non-standard version."),
-            r"tx-size": _("The transaction was rejected because it is too large (in bytes)."),
-            r"scriptsig-size": None,
-            r"scriptsig-not-pushonly": None,
-            r"scriptpubkey":
-                ("scriptpubkey\n" +
-                 _("Some of the outputs pay to a non-standard script.")),
-            r"bare-multisig": None,
-            r"dust":
-                (_("Transaction could not be broadcast due to dust outputs.\n"
-                   "Some of the outputs are too small in value, probably lower than 1000 satoshis.\n"
-                   "Check the units, make sure you haven't confused e.g. mBTC and BTC.")),
-            r"multi-op-return": _("The transaction was rejected because it contains multiple OP_RETURN outputs."),
-        }
-        for substring in policy_error_messages:
-            if substring in server_msg:
-                msg = policy_error_messages[substring]
-                return msg if msg else substring
+
         # https://github.com/bitcoin/bitcoin/blob/5bb64acd9d3ced6e6f95df282a1a0f8b98522cb0/src/script/script_error.cpp
         script_error_messages = {
             r"Script evaluated without error but finished with a false/empty top stack element",
@@ -989,7 +979,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         # https://github.com/bitcoin/bitcoin/blob/5bb64acd9d3ced6e6f95df282a1a0f8b98522cb0/src/validation.cpp
         # grep "REJECT_"
         # grep "TxValidationResult"
-        # should come after script_error.cpp (due to e.g. non-mandatory-script-verify-flag)
+        # should come after script_error.cpp (due to e.g. "non-mandatory-script-verify-flag")
         validation_error_messages = {
             r"coinbase": None,
             r"tx-size-small": None,
@@ -1071,6 +1061,29 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             if substring in server_msg:
                 msg = tx_verify_error_messages[substring]
                 return msg if msg else substring
+        # https://github.com/bitcoin/bitcoin/blob/5bb64acd9d3ced6e6f95df282a1a0f8b98522cb0/src/policy/policy.cpp
+        # grep "reason ="
+        # should come after validation.cpp (due to "tx-size" vs "tx-size-small")
+        # should come after script_error.cpp (due to e.g. "version")
+        policy_error_messages = {
+            r"version": _("Transaction uses non-standard version."),
+            r"tx-size": _("The transaction was rejected because it is too large (in bytes)."),
+            r"scriptsig-size": None,
+            r"scriptsig-not-pushonly": None,
+            r"scriptpubkey":
+                ("scriptpubkey\n" +
+                 _("Some of the outputs pay to a non-standard script.")),
+            r"bare-multisig": None,
+            r"dust":
+                (_("Transaction could not be broadcast due to dust outputs.\n"
+                   "Some of the outputs are too small in value, probably lower than 1000 satoshis.\n"
+                   "Check the units, make sure you haven't confused e.g. mBTC and BTC.")),
+            r"multi-op-return": _("The transaction was rejected because it contains multiple OP_RETURN outputs."),
+        }
+        for substring in policy_error_messages:
+            if substring in server_msg:
+                msg = policy_error_messages[substring]
+                return msg if msg else substring
         # otherwise:
         return _("Unknown error")
 
@@ -1181,7 +1194,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
     async def _start(self):
         assert not self.taskgroup
-        self.taskgroup = taskgroup = SilentTaskGroup()
+        self.taskgroup = taskgroup = OldTaskGroup()
         assert not self.interface and not self.interfaces
         assert not self._connecting_ifaces
         assert not self._closing_ifaces
@@ -1192,19 +1205,17 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         await self.taskgroup.spawn(self._run_new_interface(self.default_server))
 
         async def main():
-            self.logger.info("starting taskgroup.")
+            self.logger.info(f"starting taskgroup ({hex(id(taskgroup))}).")
             try:
                 # note: if a task finishes with CancelledError, that
                 # will NOT raise, and the group will keep the other tasks running
                 async with taskgroup as group:
                     await group.spawn(self._maintain_sessions())
                     [await group.spawn(job) for job in self._jobs]
-            except asyncio.CancelledError:
-                raise
             except Exception as e:
-                self.logger.exception("taskgroup died.")
+                self.logger.exception(f"taskgroup died ({hex(id(taskgroup))}).")
             finally:
-                self.logger.info("taskgroup stopped.")
+                self.logger.info(f"taskgroup stopped ({hex(id(taskgroup))}).")
         asyncio.run_coroutine_threadsafe(main(), self.asyncio_loop)
 
         util.trigger_callback('network_updated')
@@ -1224,7 +1235,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         # timeout: if full_shutdown, it is up to the caller to time us out,
         #          otherwise if e.g. restarting due to proxy changes, we time out fast
         async with (nullcontext() if full_shutdown else ignore_after(1)):
-            async with TaskGroup() as group:
+            async with OldTaskGroup() as group:
                 await group.spawn(self.taskgroup.cancel_remaining())
                 if full_shutdown:
                     await group.spawn(self.stop_gossip(full_shutdown=full_shutdown))
@@ -1237,13 +1248,13 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             util.trigger_callback('network_updated')
 
     async def _ensure_there_is_a_main_interface(self):
-        if self.is_connected():
+        if self.interface:
             return
         # if auto_connect is set, try a different server
         if self.auto_connect and not self.is_connecting():
             await self._switch_to_random_interface()
         # if auto_connect is not set, or still no main interface, retry current
-        if not self.is_connected() and not self.is_connecting():
+        if not self.interface and not self.is_connecting():
             if self._can_retry_addr(self.default_server, urgent=True):
                 await self.switch_to_interface(self.default_server)
 
@@ -1270,15 +1281,9 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
                     await self.interface.taskgroup.spawn(self._request_fee_estimates, self.interface)
 
         while True:
-            try:
-                await maybe_start_new_interfaces()
-                await maintain_healthy_spread_of_connected_servers()
-                await maintain_main_interface()
-            except asyncio.CancelledError:
-                # suppress spurious cancellations
-                group = self.taskgroup
-                if not group or group.closed():
-                    raise
+            await maybe_start_new_interfaces()
+            await maintain_healthy_spread_of_connected_servers()
+            await maintain_main_interface()
             await asyncio.sleep(0.1)
 
     @classmethod
@@ -1328,11 +1333,19 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         session = self.interface.session
         return parse_servers(await session.send_request('server.peers.subscribe'))
 
-    async def send_multiple_requests(self, servers: Sequence[ServerAddr], method: str, params: Sequence):
+    async def send_multiple_requests(
+            self,
+            servers: Sequence[ServerAddr],
+            method: str,
+            params: Sequence,
+            *,
+            timeout: int = None,
+    ):
+        if timeout is None:
+            timeout = self.get_network_timeout_seconds(NetworkTimeout.Urgent)
         responses = dict()
         async def get_response(server: ServerAddr):
             interface = Interface(network=self, server=server, proxy=self.proxy)
-            timeout = self.get_network_timeout_seconds(NetworkTimeout.Urgent)
             try:
                 await asyncio.wait_for(interface.ready, timeout)
             except BaseException as e:
@@ -1343,7 +1356,16 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             except Exception as e:
                 res = e
             responses[interface.server] = res
-        async with TaskGroup() as group:
+        async with OldTaskGroup() as group:
             for server in servers:
                 await group.spawn(get_response(server))
         return responses
+
+    async def prune_offline_servers(self, hostmap):
+        peers = filter_protocol(hostmap, allowed_protocols=("t", "s",))
+        timeout = self.get_network_timeout_seconds(NetworkTimeout.Generic)
+        replies = await self.send_multiple_requests(peers, 'blockchain.headers.subscribe', [], timeout=timeout)
+        servers_replied = {serveraddr.host for serveraddr in replies.keys()}
+        servers_dict = {k: v for k, v in hostmap.items()
+                        if k in servers_replied}
+        return servers_dict
