@@ -34,6 +34,12 @@ FEERATE_STATIC_VALUES = [1000, 2000, 5000, 10000, 20000, 30000,
                          50000, 70000, 100000, 150000, 200000, 300000]
 FEERATE_REGTEST_HARDCODED = 180000  # for eclair compat
 
+# The min feerate_per_kw that can be used in lightning so that
+# the resulting onchain tx pays the min relay fee.
+# This would be FEERATE_DEFAULT_RELAY / 4 if not for rounding errors,
+# see https://github.com/ElementsProject/lightning/commit/2e687b9b352c9092b5e8bd4a688916ac50b44af0
+FEERATE_PER_KW_MIN_RELAY_LIGHTNING = 253
+
 FEE_RATIO_HIGH_WARNING = 0.05  # warn user if fee/amount for on-chain tx is higher than this
 
 
@@ -110,6 +116,8 @@ class SimpleConfig(Logger):
         except UnknownBaseUnit:
             self.decimal_point = DECIMAL_POINT_DEFAULT
         self.num_zeros = int(self.get('num_zeros', 0))
+        self.amt_precision_post_satoshi = int(self.get('amt_precision_post_satoshi', 0))
+        self.amt_add_thousands_sep = bool(self.get('amt_add_thousands_sep', False))
 
     def electrum_path(self):
         # Read electrum_path from command line
@@ -665,6 +673,8 @@ class SimpleConfig(Logger):
             decimal_point=self.decimal_point,
             is_diff=is_diff,
             whitespaces=whitespaces,
+            precision=self.amt_precision_post_satoshi,
+            add_thousands_sep=self.amt_add_thousands_sep,
         )
 
     def format_amount_and_units(self, amount):
