@@ -52,7 +52,6 @@ import attr
 import aiohttp
 from aiohttp_socks import ProxyConnector, ProxyType
 import aiorpcx
-from aiorpcx import TaskGroup
 import certifi
 import dns.resolver
 
@@ -108,6 +107,31 @@ def base_unit_name_to_decimal_point(unit_name: str) -> int:
     except KeyError:
         raise UnknownBaseUnit(unit_name) from None
 
+def parse_max_spend(amt: Any) -> Optional[int]:
+    """Checks if given amount is "spend-max"-like.
+    Returns None or the positive integer weight for "max". Never raises.
+
+    When creating invoices and on-chain txs, the user can specify to send "max".
+    This is done by setting the amount to '!'. Splitting max between multiple
+    tx outputs is also possible, and custom weights (positive ints) can also be used.
+    For example, to send 40% of all coins to address1, and 60% to address2:
+    ```
+    address1, 2!
+    address2, 3!
+    ```
+    """
+    if not (isinstance(amt, str) and amt and amt[-1] == '!'):
+        return None
+    if amt == '!':
+        return 1
+    x = amt[:-1]
+    try:
+        x = int(x)
+    except ValueError:
+        return None
+    if x > 0:
+        return x
+    return None
 
 class NotEnoughFunds(Exception):
     def __str__(self):
@@ -119,11 +143,6 @@ class NoDynamicFeeEstimates(Exception):
         return _('Dynamic fee estimates not available')
 
 
-class MultipleSpendMaxTxOutputs(Exception):
-    def __str__(self):
-        return _('At most one output can be set to spend max')
-
-
 class InvalidPassword(Exception):
     def __str__(self):
         return _("Incorrect password")
@@ -320,7 +339,7 @@ class DaemonThread(threading.Thread, Logger):
     def __init__(self):
         threading.Thread.__init__(self)
         Logger.__init__(self)
-        self.parent_thread = threading.currentThread()
+        self.parent_thread = threading.current_thread()
         self.running = False
         self.running_lock = threading.Lock()
         self.job_lock = threading.Lock()
@@ -627,11 +646,16 @@ def chunks(items, size: int):
         yield items[i: i + size]
 
 
-def format_satoshis_plain(x, *, decimal_point=8) -> str:
+def format_satoshis_plain(
+        x: Union[int, float, Decimal, str],  # amount in satoshis,
+        *,
+        decimal_point: int = 8,  # how much to shift decimal point to left (default: sat->BTC)
+) -> str:
     """Display a satoshi amount scaled.  Always uses a '.' as a decimal
     point and has no thousands separator"""
-    if x == '!':
-        return 'max'
+    if parse_max_spend(x):
+        return f'max({x})'
+    assert isinstance(x, (int, float, Decimal)), f"{x!r} should be a number"
     scale_factor = pow(10, decimal_point)
     return "{:.8f}".format(Decimal(x) / scale_factor).rstrip('0').rstrip('.')
 
@@ -647,40 +671,57 @@ DECIMAL_POINT = localeconv()['decimal_point']  # type: str
 
 
 def format_satoshis(
-        x,  # in satoshis
+        x: Union[int, float, Decimal, str, None],  # amount in satoshis
         *,
-        num_zeros=0,
-        decimal_point=8,
-        precision=None,
-        is_diff=False,
-        whitespaces=False,
+        num_zeros: int = 0,
+        decimal_point: int = 8,  # how much to shift decimal point to left (default: sat->BTC)
+        precision: int = 0,  # extra digits after satoshi precision
+        is_diff: bool = False,  # if True, enforce a leading sign (+/-)
+        whitespaces: bool = False,  # if True, add whitespaces, to align numbers in a column
+        add_thousands_sep: bool = False,  # if True, add whitespaces, for better readability of the numbers
 ) -> str:
     if x is None:
         return 'unknown'
-    if x == '!':
-        return 'max'
-    if precision is None:
-        precision = decimal_point
+    if parse_max_spend(x):
+        return f'max({x})'
+    assert isinstance(x, (int, float, Decimal)), f"{x!r} should be a number"
+    # lose redundant precision
+    x = Decimal(x).quantize(Decimal(10) ** (-precision))
     # format string
-    decimal_format = "." + str(precision) if precision > 0 else ""
+    overall_precision = decimal_point + precision  # max digits after final decimal point
+    decimal_format = "." + str(overall_precision) if overall_precision > 0 else ""
     if is_diff:
         decimal_format = '+' + decimal_format
     # initial result
     scale_factor = pow(10, decimal_point)
-    if not isinstance(x, Decimal):
-        x = Decimal(x).quantize(Decimal('1E-8'))
     result = ("{:" + decimal_format + "f}").format(x / scale_factor)
     if "." not in result: result += "."
     result = result.rstrip('0')
-    # extra decimal places
+    # add extra decimal places (zeros)
     integer_part, fract_part = result.split(".")
     if len(fract_part) < num_zeros:
         fract_part += "0" * (num_zeros - len(fract_part))
+    # add whitespaces as thousands' separator for better readability of numbers
+    if add_thousands_sep:
+        sign = integer_part[0] if integer_part[0] in ("+", "-") else ""
+        if sign == "-":
+            integer_part = integer_part[1:]
+        integer_part = "{:,}".format(int(integer_part)).replace(',', " ")
+        integer_part = sign + integer_part
+        fract_part = " ".join(fract_part[i:i+3] for i in range(0, len(fract_part), 3))
     result = integer_part + DECIMAL_POINT + fract_part
-    # leading/trailing whitespaces
+    # add leading/trailing whitespaces so that numbers can be aligned in a column
     if whitespaces:
-        result += " " * (decimal_point - len(fract_part))
-        result = " " * (15 - len(result)) + result
+        target_fract_len = overall_precision
+        target_integer_len = 14 - decimal_point  # should be enough for up to unsigned 999999 BTC
+        if add_thousands_sep:
+            target_fract_len += max(0, (target_fract_len - 1) // 3)
+            target_integer_len += max(0, (target_integer_len - 1) // 3)
+        # add trailing whitespaces
+        result += " " * (target_fract_len - len(fract_part))
+        # add leading whitespaces
+        target_total_len = target_integer_len + 1 + target_fract_len
+        result = " " * (target_total_len - len(result)) + result
     return result
 
 
@@ -1070,7 +1111,8 @@ def setup_thread_excepthook():
 
 
 def send_exception_to_crash_reporter(e: BaseException):
-    sys.excepthook(type(e), e, e.__traceback__)
+    from .base_crash_reporter import send_exception_to_crash_reporter
+    send_exception_to_crash_reporter(e)
 
 
 def versiontuple(v):
@@ -1135,9 +1177,6 @@ def ignore_exceptions(func):
     async def wrapper(*args, **kwargs):
         try:
             return await func(*args, **kwargs)
-        except asyncio.CancelledError:
-            # note: with python 3.8, CancelledError no longer inherits Exception, so this catch is redundant
-            raise
         except Exception as e:
             pass
     return wrapper
@@ -1186,13 +1225,76 @@ def make_aiohttp_session(proxy: Optional[dict], headers=None, timeout=None):
     return aiohttp.ClientSession(headers=headers, timeout=timeout, connector=connector)
 
 
-class SilentTaskGroup(TaskGroup):
+class OldTaskGroup(aiorpcx.TaskGroup):
+    """Automatically raises exceptions on join; as in aiorpcx prior to version 0.20.
+    That is, when using TaskGroup as a context manager, if any task encounters an exception,
+    we would like that exception to be re-raised (propagated out). For the wait=all case,
+    the OldTaskGroup class is emulating the following code-snippet:
+    ```
+    async with TaskGroup() as group:
+        await group.spawn(task1())
+        await group.spawn(task2())
 
-    def spawn(self, *args, **kwargs):
-        # don't complain if group is already closed.
-        if self._closed:
-            raise asyncio.CancelledError()
-        return super().spawn(*args, **kwargs)
+        async for task in group:
+            if not task.cancelled():
+                task.result()
+    ```
+    So instead of the above, one can just write:
+    ```
+    async with OldTaskGroup() as group:
+        await group.spawn(task1())
+        await group.spawn(task2())
+    ```
+    """
+    async def join(self):
+        if self._wait is all:
+            exc = False
+            try:
+                async for task in self:
+                    if not task.cancelled():
+                        task.result()
+            except BaseException:  # including asyncio.CancelledError
+                exc = True
+                raise
+            finally:
+                if exc:
+                    await self.cancel_remaining()
+                await super().join()
+        else:
+            await super().join()
+            if self.completed:
+                self.completed.result()
+
+# We monkey-patch aiorpcx TimeoutAfter (used by timeout_after and ignore_after API),
+# to fix a timing issue present in asyncio as a whole re timing out tasks.
+# To see the issue we are trying to fix, consider example:
+#     async def outer_task():
+#         async with timeout_after(0.1):
+#             await inner_task()
+# When the 0.1 sec timeout expires, inner_task will get cancelled by timeout_after (=internal cancellation).
+# If around the same time (in terms of event loop iterations) another coroutine
+# cancels outer_task (=external cancellation), there will be a race.
+# Both cancellations work by propagating a CancelledError out to timeout_after, which then
+# needs to decide (in TimeoutAfter.__aexit__) whether it's due to an internal or external cancellation.
+# AFAICT asyncio provides no reliable way of distinguishing between the two.
+# This patch tries to always give priority to external cancellations.
+# see https://github.com/kyuupichan/aiorpcX/issues/44
+# see https://github.com/aio-libs/async-timeout/issues/229
+# see https://bugs.python.org/issue42130 and https://bugs.python.org/issue45098
+def _aiorpcx_monkeypatched_set_new_deadline(task, deadline):
+    def timeout_task():
+        task._orig_cancel()
+        task._timed_out = None if getattr(task, "_externally_cancelled", False) else deadline
+    def mycancel(*args, **kwargs):
+        task._orig_cancel(*args, **kwargs)
+        task._externally_cancelled = True
+        task._timed_out = None
+    if not hasattr(task, "_orig_cancel"):
+        task._orig_cancel = task.cancel
+        task.cancel = mycancel
+    task._deadline_handle = task._loop.call_at(deadline, timeout_task)
+
+aiorpcx.curio._set_new_deadline = _aiorpcx_monkeypatched_set_new_deadline
 
 
 class NetworkJobOnDefaultServer(Logger, ABC):
@@ -1220,14 +1322,14 @@ class NetworkJobOnDefaultServer(Logger, ABC):
         """Initialise fields. Called every time the underlying
         server connection changes.
         """
-        self.taskgroup = SilentTaskGroup()
+        self.taskgroup = OldTaskGroup()
 
     async def _start(self, interface: 'Interface'):
         self.interface = interface
         await interface.taskgroup.spawn(self._run_tasks(taskgroup=self.taskgroup))
 
     @abstractmethod
-    async def _run_tasks(self, *, taskgroup: TaskGroup) -> None:
+    async def _run_tasks(self, *, taskgroup: OldTaskGroup) -> None:
         """Start tasks in taskgroup. Called every time the underlying
         server connection changes.
         """
@@ -1274,7 +1376,7 @@ def create_and_start_event_loop() -> Tuple[asyncio.AbstractEventLoop,
     loop = asyncio.get_event_loop()
     loop.set_exception_handler(on_exception)
     # loop.set_debug(1)
-    stopping_fut = asyncio.Future()
+    stopping_fut = loop.create_future()
     loop_thread = threading.Thread(target=loop.run_until_complete,
                                          args=(stopping_fut,),
                                          name='EventLoop')
@@ -1626,10 +1728,8 @@ class nullcontext:
     async def __aexit__(self, *excinfo):
         pass
 
-def get_running_loop():
-    """Mimics _get_running_loop convenient functionality for sanity checks on all python versions"""
-    if sys.version_info < (3, 7):
-        return asyncio._get_running_loop()
+
+def get_running_loop() -> Optional[asyncio.AbstractEventLoop]:
     try:
         return asyncio.get_running_loop()
     except RuntimeError:
