@@ -33,11 +33,11 @@ import binascii
 
 from . import util, bitcoin
 from .util import profiler, WalletFileException, multisig_type, TxMinedInfo, bfh
-from .invoices import PR_TYPE_ONCHAIN, Invoice
+from .invoices import Invoice
 from .keystore import bip44_derivation
 from .transaction import Transaction, TxOutpoint, tx_from_any, PartialTransaction, PartialTxOutput
 from .logging import Logger
-from .lnutil import LOCAL, REMOTE, FeeUpdate, UpdateAddHtlc, LocalConfig, RemoteConfig, Keypair, OnlyPubkeyKeypair, RevocationStore
+from .lnutil import LOCAL, REMOTE, FeeUpdate, UpdateAddHtlc, LocalConfig, RemoteConfig, ChannelType
 from .lnutil import ImportedChannelBackupStorage, OnchainChannelBackupStorage
 from .lnutil import ChannelConstraints, Outpoint, ShachainElement
 from .json_db import StoredDict, JsonDB, locked, modifier
@@ -53,7 +53,7 @@ if TYPE_CHECKING:
 
 OLD_SEED_VERSION = 4        # electrum versions < 2.0
 NEW_SEED_VERSION = 11       # electrum versions >= 2.0
-FINAL_SEED_VERSION = 41     # electrum >= 2.7 will set this to prevent
+FINAL_SEED_VERSION = 44     # electrum >= 2.7 will set this to prevent
                             # old versions from overwriting new format
 
 
@@ -190,6 +190,9 @@ class WalletDB(JsonDB):
         self._convert_version_39()
         self._convert_version_40()
         self._convert_version_41()
+        self._convert_version_42()
+        self._convert_version_43()
+        self._convert_version_44()
         self.put('seed_version', FINAL_SEED_VERSION)  # just to be sure
 
         self._after_upgrade_tasks()
@@ -557,6 +560,7 @@ class WalletDB(JsonDB):
         if not self._is_upgrade_method_needed(24, 24):
             return
         # add 'type' field to onchain requests
+        PR_TYPE_ONCHAIN = 0
         requests = self.data.get('payment_requests', {})
         for k, r in list(requests.items()):
             if r.get('address') == k:
@@ -624,6 +628,7 @@ class WalletDB(JsonDB):
     def _convert_version_29(self):
         if not self._is_upgrade_method_needed(28, 28):
             return
+        PR_TYPE_ONCHAIN = 0
         requests = self.data.get('payment_requests', {})
         invoices = self.data.get('invoices', {})
         for d in [invoices, requests]:
@@ -659,8 +664,8 @@ class WalletDB(JsonDB):
     def _convert_version_30(self):
         if not self._is_upgrade_method_needed(29, 29):
             return
-
-        from .invoices import PR_TYPE_ONCHAIN, PR_TYPE_LN
+        PR_TYPE_ONCHAIN = 0
+        PR_TYPE_LN = 2
         requests = self.data.get('payment_requests', {})
         invoices = self.data.get('invoices', {})
         for d in [invoices, requests]:
@@ -682,8 +687,7 @@ class WalletDB(JsonDB):
     def _convert_version_31(self):
         if not self._is_upgrade_method_needed(30, 30):
             return
-
-        from .invoices import PR_TYPE_ONCHAIN
+        PR_TYPE_ONCHAIN = 0
         requests = self.data.get('payment_requests', {})
         invoices = self.data.get('invoices', {})
         for d in [invoices, requests]:
@@ -821,6 +825,45 @@ class WalletDB(JsonDB):
         self.data['imported_channel_backups'] = imported_channel_backups
         self.data['seed_version'] = 41
 
+    def _convert_version_42(self):
+        # in OnchainInvoice['outputs'], convert values from None to 0
+        if not self._is_upgrade_method_needed(41, 41):
+            return
+        PR_TYPE_ONCHAIN = 0
+        requests = self.data.get('payment_requests', {})
+        invoices = self.data.get('invoices', {})
+        for d in [invoices, requests]:
+            for key, item in list(d.items()):
+                if item['type'] == PR_TYPE_ONCHAIN:
+                    item['outputs'] = [(_type, addr, (val or 0))
+                                       for _type, addr, val in item['outputs']]
+        self.data['seed_version'] = 42
+
+    def _convert_version_43(self):
+        if not self._is_upgrade_method_needed(42, 42):
+            return
+        channels = self.data.pop('channels', {})
+        for k, c in channels.items():
+            log = c['log']
+            c['fail_htlc_reasons'] = log.pop('fail_htlc_reasons', {})
+            c['unfulfilled_htlcs'] = log.pop('unfulfilled_htlcs', {})
+            log["1"]['unacked_updates'] = log.pop('unacked_local_updates2', {})
+        self.data['channels'] = channels
+        self.data['seed_version'] = 43
+
+    def _convert_version_44(self):
+        if not self._is_upgrade_method_needed(43, 43):
+            return
+        channels = self.data.get('channels', {})
+        for key, item in channels.items():
+            if bool(item.get('static_remotekey_enabled')):
+                channel_type = ChannelType.OPTION_STATIC_REMOTEKEY
+            else:
+                channel_type = ChannelType(0)
+            item.pop('static_remotekey_enabled', None)
+            item['channel_type'] = channel_type
+        self.data['seed_version'] = 44
+
     def _convert_imported(self):
         if not self._is_upgrade_method_needed(0, 13):
             return
@@ -1328,6 +1371,15 @@ class WalletDB(JsonDB):
             v = dict((k, ShachainElement(bfh(x[0]), int(x[1]))) for k, x in v.items())
         elif key == 'data_loss_protect_remote_pcp':
             v = dict((k, bfh(x)) for k, x in v.items())
+        # convert htlc_id keys to int
+        if key in ['adds', 'locked_in', 'settles', 'fails', 'fee_updates', 'buckets',
+                   'unacked_updates', 'unfulfilled_htlcs', 'fail_htlc_reasons', 'onion_keys']:
+            v = dict((int(k), x) for k, x in v.items())
+        # convert keys to HTLCOwner
+        if key == 'log' or (path and path[-1] in ['locked_in', 'fails', 'settles']):
+            if "1" in v:
+                v[LOCAL] = v.pop("1")
+                v[REMOTE] = v.pop("-1")
         return v
 
     def _convert_value(self, path, key, v):
@@ -1339,6 +1391,8 @@ class WalletDB(JsonDB):
             v = ChannelConstraints(**v)
         elif key == 'funding_outpoint':
             v = Outpoint(**v)
+        elif key == 'channel_type':
+            v = ChannelType(v)
         return v
 
     def _should_convert_to_stored_dict(self, key) -> bool:
@@ -1355,7 +1409,7 @@ class WalletDB(JsonDB):
 
     @profiler
     def _write(self, storage: 'WalletStorage'):
-        if threading.currentThread().isDaemon():
+        if threading.current_thread().daemon:
             self.logger.warning('daemon thread cannot write db')
             return
         if not self.modified():
